@@ -10,14 +10,15 @@ Written in python
 
 """
 
-__title__ = 'askitsu'
-__author__ = 'ShomyKohai'
-__license__ = 'MIT'
-__copyright__ = 'Copyright 2022-present ShomyKohai'
-__version__ = '0.4.0'
+__title__ = "askitsu"
+__author__ = "ShomyKohai"
+__license__ = "MIT"
+__copyright__ = "Copyright 2022-present ShomyKohai"
+__version__ = "0.4.1"
 
 from .anime import *
 from .client import *
 from .core import *
 from .error import *
+from .images import *
 from .manga import *
