@@ -25,11 +25,15 @@ DEALINGS IN THE SOFTWARE.
 from __future__ import annotations
 
 from datetime import datetime
-from typing import Optional, Literal
+from typing import Literal, Optional
+from .images import Image
+
 
 class Character:
     """Represents a :class:`Character` istance
 
+    .. versionadded:: 0.2.0
+
     Attributes
     -----------
     media_id: :class:`int`
@@ -50,53 +54,45 @@ class Character:
         Return character's image
     created_at: Optional[:class:`datetime`]
     updated_at: Optional[:class:`datetime`]
-    """       
+    """
 
-    __slots__ = ('id', 'type', 'media_id', 'name', 'slug', 'description', 'role',
-                'slug', 'mal_id', 'image', 'created_at', 'updated_at')
+    __slots__ = (
+        "id",
+        "type",
+        "media_id",
+        "name",
+        "slug",
+        "description",
+        "role",
+        "slug",
+        "mal_id",
+        "created_at",
+        "updated_at",
+        "_data"
+    )
 
-    def __init__(self, attributes: dict, *, entry_id: str = None):
+    def __init__(self, attributes: dict, *, role: str, entry_id: int = None):
+        data = attributes["attributes"]
+        self._data = data
         self.media_id = entry_id
-        data = attributes['attributes']
-        self.id: str = attributes['id']
+        self.id: int = int(attributes["id"])
         self.type: str = "characters"
-        self.name: str = data['canonicalName']
-        self.description: str = data['description']
-        self.role: Literal["main", "supporting"] = None
-        self.slug: str = data['slug']
-        self.mal_id: str = data['malId']
-        self.created_at: datetime = datetime.strptime(data['createdAt'], "%Y-%m-%dT%H:%M:%S.%fZ") if (
-            data['createdAt']) else None
-        self.updated_at: datetime = datetime.strptime(data['updatedAt'], "%Y-%m-%dT%H:%M:%S.%fZ") if (
-            data['updatedAt']) else None
-        self.image: str = data['image']
+        self.name: str = data["canonicalName"]
+        self.description: str = data["description"]
+        self.role: str = role
+        self.slug: str = data["slug"]
+        self.mal_id: int = int(data["malId"])
+        self.created_at: Optional[datetime] = (
+            datetime.strptime(data["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
+            if (data["createdAt"])
+            else None
+        )
+        self.updated_at: Optional[datetime] = (
+            datetime.strptime(data["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
+            if (data["updatedAt"])
+            else None
+        )
 
-
-    @classmethod
-    async def _character_instance(
-            cls, 
-            link: str = None,
-            _cls = None, 
-            entry_id: str = None, 
-            role: Literal["main", "supporting"] = None
-    ) -> Character:
-        _data = await _cls._get_data(url=link)
-        _character = Character(_data["data"], entry_id)
-        _character.role = role
-        return _character
-
-    def get_image(self, size: 
-        Optional[Literal["tiny", "small", "medium", "large", "original"]] = "original"
-    ) -> Optional[str]:
-        """Get character image
-
-        Parameters
-        -----------
-            size: Optional[Literal["tiny", "small", "medium", "large", "original"]]
-                Size of the cover image
-        """
-        try: 
-            return self.image.get(size, None)
-        except AttributeError:
-            return None
-        
\ No newline at end of file
+    @property
+    def image(self) -> Image:
+        return Image(self._data["image"])
