@@ -26,16 +26,23 @@ import aiohttp
 from colorama import Fore, Style
 from typing import (
     Any, 
-    Optional, 
-    Union, 
     List,
-    Literal
+    Literal,
+    Optional,
+    overload,
+    Union
 )
 from .anime import Anime, StreamLink
 from .character import Character
 from .core import Review, BASE
+from .error import (
+    BadApiRequest, 
+    InvalidArgument,
+    NotAuthenticated
+)
+from . import __version__
 from .manga import Manga
-from .error import *
+
 
 
 class Client:
@@ -43,25 +50,39 @@ class Client:
 
     Parameters
     -----------
+    token: :class:`str`
+        Access token to make authenticated requests.\n
+        Useful when you need to fetch NSFW content, interact with users or post something. 
+
+        .. versionadded:: 0.4.1
+
     session: Optional[:class:`aiohttp.ClientSession`]
         An object that represents the effective connection
-
+    
+    Attributes
+    -----------
+    token: :class:`str`
+        Token passed to the session.
     """
 
-    def __init__(self, session: Optional[aiohttp.ClientSession] = None) -> None:
+    def __init__(self, token: str = None, *, session: Optional[aiohttp.ClientSession] = None) -> None:
         self._entries = {
             "anime": Anime,
             "manga": Manga,
             "characters": Character
         }
+        self.__authorization = f"Bearer {token}" if token else ""
         self._session: aiohttp.ClientSession = session or aiohttp.ClientSession()
-        self._headers = {
+        self.__headers = {
             "Accept": "application/vnd.api+json",
             "Content-Type": "application/vnd.api+json",
+            "User-Agent": f"askitsu (https://github.com/ShomyKohai/askitsu {__version__})",
+            "Authorization": self.__authorization
         }
+        self.token = token
 
     async def _get_data(self, url: str) -> Any:
-        async with self._session.get(url=url, headers=self._headers) as response:
+        async with self._session.get(url=url, headers=self.__headers) as response:
             response_data = await response.json()
             if response.status == 200:
                 return response_data
@@ -72,16 +93,48 @@ class Client:
             if response.status == 400:
                 raise BadApiRequest(response_data["errors"][0])
 
+    @overload
     async def search(
-        self, 
-        type: Literal["anime", "manga", "characters"], 
-        query: str, 
-        limit: int = 1
-    ) -> Optional[Union[
-            Anime, List[Anime], 
-            Manga, List[Manga],
-            Character, List[Character]
-        ]]:
+       self, type: Literal["anime"], query: str
+    ) -> Optional[Anime]:
+        ...
+    
+    @overload
+    async def search(
+       self, type: Literal["anime"], query: str, limit: int = ...
+    ) -> Optional[List[Anime]]:
+        ...
+
+    @overload
+    async def search(
+       self, type: Literal["manga"], query: str
+    ) -> Optional[Manga]:
+        ...
+    
+    @overload
+    async def search(
+       self, type: Literal["manga"], query: str, limit: int = ...
+    ) -> Optional[List[Manga]]:
+        ...
+    
+    @overload
+    async def search(
+       self, type: Literal["characters"], query: str
+    ) -> Optional[Character]:
+        ...
+    
+    @overload
+    async def search(
+       self, type: Literal["characters"], query: str, limit: int = ...
+    ) -> Optional[List[Character]]:
+        ...    
+       
+
+    async def search(
+        self, type: Literal["anime", "manga", "characters"], query: str, limit: int = 1
+    ) -> Optional[
+        Union[Anime, List[Anime], Manga, List[Manga], Character, List[Character]]
+    ]:
         """|coro|
 
         Search trough Kitsu API with the providen query and fetch the found data
@@ -111,6 +164,18 @@ class Client:
             for attributes in fetched_data["data"]
         ]
 
+    @overload
+    async def search_anime(
+        self, query: str
+    ) -> Optional[Anime]:
+        ...
+
+    @overload
+    async def search_anime(
+        self, query: str, limit: int = ...
+    ) -> Optional[List[Anime]]:
+        ...
+   
     async def search_anime(
         self, query: str, limit: int = 1
     ) -> Optional[Union[Anime, List[Anime]]]:
@@ -127,6 +192,17 @@ class Client:
         """
         return await self.search("anime", query=query, limit=limit)
 
+    @overload
+    async def search_manga(
+        self, query: str
+    ) -> Optional[Manga]:
+        ...
+    @overload
+    async def search_manga(
+        self, query: str, limit: int = ...
+    ) -> Optional[List[Manga]]:
+        ...    
+
     async def search_manga(
         self, query: str, limit: int = 1
     ) -> Optional[Union[Manga, List[Manga]]]:
@@ -143,6 +219,18 @@ class Client:
         """
         return await self.search("manga", query=query, limit=limit)
 
+    @overload
+    async def search_character(
+        self, query: str
+    ) -> Optional[Character]:
+        ...
+
+    @overload
+    async def search_character(
+        self, query: str, limit: int = ...
+    ) -> Optional[List[Character]]:
+        ...
+
     async def search_character(
         self, query: str, limit: int = 1
     ) -> Optional[Union[Character, List[Character]]]:
@@ -150,20 +238,43 @@ class Client:
 
         Shortcut function to :meth:`search` with the `type` parameter populated by the "character" keyword
 
+        .. versionadded:: 0.3.0
+
         Parameters
         -----------
         query: :class:`str`
             Represents the search query
         limit: :class:`int`
             Limit the search to a specific number of results
-        
+
         Note
         ------------
         By searching characters, you will not get :attr:`askitsu.Character.media_id` and :attr:`askitsu.Character.role` attributes
         """
         return await self.search("characters", query=query, limit=limit)
 
-    async def get_entry(self, type: str, id: int) -> Union[Anime, Manga, Character]:
+
+    @overload
+    async def get_entry(
+        self, type: Literal["anime"], id: int
+    ) -> Anime:
+        ...
+
+    @overload
+    async def get_entry(
+        self, type: Literal["manga"], id: int
+    ) -> Manga:
+        ... 
+
+    @overload
+    async def get_entry(
+        self, type: Literal["characters"], id: int
+    ) -> Character:
+        ...
+
+    async def get_entry(
+        self, type: Literal["anime", "manga", "characters"], id: int
+    ) -> Union[Anime, Manga, Character]:
         """|coro|
 
         Get an entry object (`Anime` | `Manga` | `Character`) by an id
@@ -178,7 +289,11 @@ class Client:
         type_lower = type.lower()
         entry = self._entries.get(type_lower)
         fetched_data = await self._get_data(f"{BASE}/{type_lower}/{id}")
-        return entry(attributes=fetched_data["data"], session=self._session) if fetched_data else None
+        return (
+            entry(attributes=fetched_data["data"], session=self._session)
+            if fetched_data
+            else None
+        )
 
     async def get_anime_entry(self, id: int) -> Anime:
         """|coro|
@@ -197,6 +312,8 @@ class Client:
 
         Shortcut for :meth:`get_entry`; returns only `Manga` object
 
+        .. versionadded:: 0.3.0
+
         Parameters
         -----------
         id: :class:`int`
@@ -224,12 +341,25 @@ class Client:
         )
         return [StreamLink(links) for links in fetched_data["data"]]
 
+
+    @overload
+    async def get_characters(
+        self, entry: Union[Anime, Manga]
+    ) -> Character:
+        ...
+
+    @overload
+    async def get_characters(
+        self, entry: Union[Anime, Manga], limit: int = ...
+    ) -> List[Character]:
+        ...
+
     async def get_characters(
         self, entry: Union[Anime, Manga], limit: int = 1
     ) -> Union[Character, List[Character]]:
         """|coro|
 
-        Return a :class:`Character` | List[:class:`Character`] 
+        Return a :class:`Character` | List[:class:`Character`]
 
         Parameters
         -----------
@@ -239,32 +369,36 @@ class Client:
             Number of characters' fetch
         """
         fetched_data = await self._get_data(
-            url=f"{BASE}/{entry.type}/{entry.id}/characters?page%5Blimit%5D={limit}"
+            url=f"{BASE}/{entry.entry_type}/{entry.id}/characters?include=character&page%5Blimit%5D={limit}"
         )
-        characters_roles = [
-            link["attributes"]["role"]
-            for link in fetched_data["data"]
+        characters_roles = [link["attributes"]["role"] for link in fetched_data["data"]]
+        characters = [
+            Character(attributes, role=role, entry_id=entry.id) 
+            for attributes, role in zip(fetched_data["included"], characters_roles)
         ]
-        characters_link = [
-            link["relationships"]["character"]["links"]["related"]
-            for link in fetched_data["data"]
-        ]
-        characters = [await Character._character_instance(
-            links, 
-            self, 
-            entry.id, 
-            role) 
-            for links, role in zip(characters_link, characters_roles)
-        ]
-        return characters if len(characters)>1 else characters[0]
+        return characters if len(characters) > 1 else characters[0]
 
+    @overload
+    async def get_trending_entry(
+        self, type: Literal["anime"]
+    ) -> Optional[List[Anime]]:
+        ...
+
+    @overload
+    async def get_trending_entry(
+        self, type: Literal["manga"]
+    ) -> Optional[List[Manga]]:
+        ...
+    
     async def get_trending_entry(
         self, type: Literal["anime", "manga"]
-    ) -> Union[List[Anime], List[Manga]]:
+    ) -> Union[List[Anime], List[Manga], None]:
         """|coro|
 
         Return a list of anime or manga
 
+        .. versionadded:: 0.2.1
+
         Parameters
         -----------
         entry: Union[:class:`Anime`, :class:`Manga`]
@@ -277,21 +411,35 @@ class Client:
                 f"Please pass 'anime' or 'manga' as parameter to {Fore.LIGHTCYAN_EX}get_trending_entry{Style.RESET_ALL}"
             )
         entry = self._entries.get(type_lower)
-        fetched_data = await self._get_data(
-            f"{BASE}/trending/{type_lower}"
-        )
-        return [entry(attributes=attributes, session=self._session) 
+        fetched_data = await self._get_data(f"{BASE}/trending/{type_lower}")
+        return [
+            entry(attributes=attributes, session=self._session)
             for attributes in fetched_data["data"]
         ]
-        
+
+
+    @overload
     async def get_reviews(
-        self, entry: Union[Manga, Anime],
-        limit: int = 1
+        self, 
+        entry: Union[Manga, Anime]
+    ) -> Optional[Review]:
+        ...
+
+    @overload
+    async def get_reviews(
+        self, entry: Union[Manga, Anime], limit: int = ...
+    ) -> Optional[List[Review]]:
+        ...
+
+    async def get_reviews(
+        self, entry: Union[Manga, Anime], limit: int = 1
     ) -> Optional[Union[Review, List[Review]]]:
         """|coro|
 
         Get reviews of a given entry
 
+        .. versionadded:: 0.3.0
+
         Parameters
         -----------
         entry: Union[Manga, Anime]
@@ -303,10 +451,10 @@ class Client:
             url=f"{BASE}/{entry.entry_type}/{entry.id}/reviews?page%5Blimit%5D={limit}"
         )
         reviews = [
-            Review(entry.id, entry.entry_type, reviews) 
+            Review(entry.id, entry.entry_type, reviews)
             for reviews in fetched_data["data"]
         ]
-        return (reviews if limit>1 else reviews[0]) if reviews else None
+        return (reviews if limit > 1 else reviews[0]) if reviews else None
 
     async def close(self) -> None:
         """
